@@ -15,7 +15,7 @@ __date__ = "Jun 27, 2012"
 
 
 from pymatgen.phasediagram.entries import PDEntry
-from pymatgen.core.structure import Composition
+from pymatgen.core.composition import Composition
 from pymatgen.serializers.json_coders import MSONable
 from pymatgen.analysis.thermochemistry import ThermoData
 
@@ -71,10 +71,9 @@ class ExpEntry(PDEntry, MSONable):
 
     @property
     def to_dict(self):
-        d = {}
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
-        d["thermodata"] = [td.to_dict for td in self._thermodata]
-        d["composition"] = self.composition.to_dict
-        d["temperature"] = self.temperature
-        return d
+        return {"@module": self.__class__.__module__,
+                "@class": self.__class__.__name__,
+                "thermodata": [td.to_dict for td in self._thermodata],
+                "composition": self.composition.to_dict,
+                "temperature": self.temperature}
+        
\ No newline at end of file
